# ast-flow-graph

[![Coveralls Status][coveralls-image]][coveralls-url]
[![Build Status][travis-image]][travis-url]
[![Dependency Status][depstat-image]][depstat-url]
[![npm version][npm-image]][npm-url]
[![License][license-image]][license-url]
[![Known Vulnerabilities][snyk-image]][snyk-url]
[![david-dm][david-dm-image]][david-dm-url]

> Constructs a CFG for JavaScript source code.

## Install

```sh
npm i ast-flow-graph
```

## Usage

```js
const 
    CFG = require( 'ast-flow-graph' ),
    fs = require( 'fs' ),
    src = fs.readFileSync( 'some-javascript.js', 'utf8' ),
    cfg = new CFG( src, {
        parser:    {
            loc:          true,
            range:        true,
            comment:      true,
            tokens:       true,
            ecmaVersion:  9,
            sourceType:   'module',
            ecmaFeatures: {
                impliedStrict: true,
                experimentalObjectRestSpread: true
            }
        }
    } );

cfg.generate();     // Generate a CFG for all functions (and main module)
// or for just one function
const graph = cfg.generate( 'my_function' );

// Create all graphs and then get one of them
cfg.generate(); // You only need to do this once.
const myFunc = cfg.by_name( 'my_function' );
// ...
console.log( cfg.toTable() );    // Display all functions as tables
// Create a graph-viz .dot file

console.log( cfg.create_dot( myFunc ) );
```

## CLI Usage

      cfg version 1.0.0

    Usage

      cfg [-d] [-g] [-s ...sources] [-t] [-n ...names] [-r] [-v] [-h] [...files]

    Options

      -d, --debug             Turn on debugging mode. Warning: This will generate a lot of output.
      -g, --graph             Create a .dot file for graph-viz
      -o, --output string     If this option is present, save the .dot files to this directory.
      -s, --source string[]   Input source file. Can also be specified at the end of the command line.
      -t, --table             Output a table showing all CFG blocks
      -l, --lines             Output CFG blocks as text
      -n, --name string[]     Specify a function name to only display information for that function.
      -v, --verbose           Output additional information
      -h, --help              Display this help message

    Description

      Creates a CFG from one or more source files.

## Plugins

This section is a work in progress. The `headers`, `rows`, and `json` are not implemented, the others are but not tested.

#### General

-   `postLoad` for general initialization
-   `preExit` for general cleanup

#### `CFG`

-   `init` before anything is done
-   `postInit` after `AST` but before graph
-   `finish` after graph as part of block finishing
-   `headers`
-   `rows`
-   `json`

#### `AST`

-   `init` before parsing
-   `postInit` after parsing
-   `finish` after parsing and traversal
-   `postFinish` after all `AST` functions are done

#### `CFGBlock`

-   `init` after the block is initialized
-   `finish` after the block has been visited
-   `postFinish` after the block has been added and the walker is moving on
-   `headers`
-   `rows`
-   `json`

#### `BlockManager`

-   `init` after the `BlockManager` has been initialized
-   `finish` after the `BlockManager` has added all blocks
-   `postFinish` after the `clean()` stage
-   `headers`
-   `rows`
-   `json`

#### Other

-   `parse` called to parse the source code

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

-   [load](#load)
-   [CFG](#cfg)
    -   [toString](#tostring)
    -   [toTable](#totable)
    -   [generate](#generate)
    -   [by_name](#by_name)
    -   [iterator](#iterator)
    -   [forEach](#foreach)
    -   [create_dot](#create_dot)
-   [AST](#ast)
    -   [node_to_scope](#node_to_scope)
    -   [forFunctions](#forfunctions)
    -   [traverse](#traverse)
    -   [walker](#walker)
    -   [flat_walker](#flat_walker)
    -   [call_visitors](#call_visitors)
    -   [add_line](#add_line)
    -   [rename](#rename)
    -   [as_source](#as_source)
    -   [get_from_function](#get_from_function)
-   [BlockManager](#blockmanager)
    -   [blocks](#blocks)
    -   [forEach](#foreach-1)
    -   [map](#map)
    -   [get](#get)
    -   [toString](#tostring-1)
    -   [toTable](#totable-1)
    -   [iterator](#iterator-1)
    -   [create_dot](#create_dot-1)
-   [CFGBlock](#cfgblock)
    -   [nodes](#nodes)
    -   [succesors_as_indices](#succesors_as_indices)
    -   [successors](#successors)
    -   [succs](#succs)
    -   [preds](#preds)
    -   [isEmpty](#isempty)
    -   [classify](#classify)
    -   [follows](#follows)
    -   [from](#from)
    -   [to](#to)
    -   [remove_succs](#remove_succs)
    -   [remove_succ](#remove_succ)
    -   [as](#as)
    -   [edge_as](#edge_as)
    -   [not](#not)
    -   [whenTrue](#whentrue)
    -   [whenFalse](#whenfalse)
    -   [add](#add)
    -   [first](#first)
    -   [last](#last)
    -   [by](#by)
    -   [isa](#isa)
    -   [eliminate](#eliminate)
    -   [defer_edge_type](#defer_edge_type)
    -   [graph_label](#graph_label)
    -   [lines](#lines)
    -   [pred_edge_types](#pred_edge_types)
    -   [succ_edge_types](#succ_edge_types)
    -   [split_by](#split_by)
    -   [toRow](#torow)
    -   [toString](#tostring-2)
-   [Edges](#edges)
    -   [reindex](#reindex)
    -   [add](#add-1)
    -   [classify](#classify-1)
    -   [not](#not-1)
    -   [retarget_multiple](#retarget_multiple)
    -   [remove_succ](#remove_succ-1)
    -   [get_succs](#get_succs)
    -   [get_preds](#get_preds)
    -   [renumber](#renumber)
    -   [successors](#successors-1)
    -   [has](#has)
    -   [edges](#edges-1)
    -   [preds](#preds-1)
    -   [pred_edges](#pred_edges)
    -   [The for Statement](#the-for-statement)
    -   [The if Statement](#the-if-statement)
    -   [The switch Statement](#the-switch-statement)
-   [Block](#block)
-   [Edge](#edge)
-   [defaultOutputOptions](#defaultoutputoptions)
-   [output](#output)
-   [pluginManager](#pluginmanager)
-   [callback](#callback)

### 

[index.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/index.js#L7-L7 "Source code on GitHub")

Description of file here.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### load

[index.js:25-35](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/index.js#L25-L35 "Source code on GitHub")

**Parameters**

-   `s` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `o` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)?** 

Returns **[CFG](#cfg)** 

### 

[src/cfg.js:8-8](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L8-L8 "Source code on GitHub")

The main CFG definition class. Generally, you start here.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### CFG

[src/cfg.js:36-152](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L36-L152 "Source code on GitHub")

**Parameters**

-   `source` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `options` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)?**  (optional, default `defaultOptions`)

#### toString

[src/cfg.js:64-67](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L64-L67 "Source code on GitHub")

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### toTable

[src/cfg.js:72-75](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L72-L75 "Source code on GitHub")

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### generate

[src/cfg.js:81-116](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L81-L116 "Source code on GitHub")

**Parameters**

-   `name` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** 

Returns **([Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;CFGInfo> | [CFG](#cfg))** 

#### by_name

[src/cfg.js:122-125](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L122-L125 "Source code on GitHub")

**Parameters**

-   `name` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

Returns **CFGInfo** 

#### iterator

[src/cfg.js:130-133](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L130-L133 "Source code on GitHub")

#### forEach

[src/cfg.js:138-141](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L138-L141 "Source code on GitHub")

**Parameters**

-   `fn` **function (CFGInfo, [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

#### create_dot

[src/cfg.js:148-151](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/cfg.js#L148-L151 "Source code on GitHub")

**Parameters**

-   `cfg` **CFGInfo** 
-   `title` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?**  (optional, default `cfg.name+':'+cfg.lines.join('-')`)

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

### 

[src/ast.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L7-L7 "Source code on GitHub")

Manages the AST, does some scans, and provides walkers.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### AST

[src/ast.js:42-490](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L42-L490 "Source code on GitHub")

**Parameters**

-   `source` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `options` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 

#### node_to_scope

[src/ast.js:147-160](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L147-L160 "Source code on GitHub")

**Parameters**

-   `node` **AnnotatedNode** 

Returns **any** 

#### forFunctions

[src/ast.js:165-168](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L165-L168 "Source code on GitHub")

#### traverse

[src/ast.js:201-218](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L201-L218 "Source code on GitHub")

**Parameters**

-   `ast` **([Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling) \| [function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function))** 
-   `enter` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)??** 
-   `leave` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)??** 

#### walker

[src/ast.js:225-269](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L225-L269 "Source code on GitHub")

**Parameters**

-   `node` **(AnnotatedNode | BaseNode | [Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling))** 
-   `enter` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)?**  (optional, default `()=>{}`)
-   `leave` **[function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)?**  (optional, default `()=>{}`)

#### flat_walker

[src/ast.js:277-291](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L277-L291 "Source code on GitHub")

Iterate over all nodes in a block without recursing into sub-nodes.

**Parameters**

-   `nodes` **([Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;AnnotatedNode> | AnnotatedNode)** 
-   `cb` **function (AnnotatedNode, function (AnnotatedNode): [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)): [boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** 

#### call_visitors

[src/ast.js:299-304](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L299-L304 "Source code on GitHub")

Callback for each visitor key for a given node.

**Parameters**

-   `node` **AnnotatedNode** 
-   `cb`  

#### add_line

[src/ast.js:312-330](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L312-L330 "Source code on GitHub")

Add a new line to the source code.

**Parameters**

-   `lineNumber` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 0-based line number
-   `sourceLine` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The source line to add

#### rename

[src/ast.js:336-342](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L336-L342 "Source code on GitHub")

**Parameters**

-   `inode` **(Identifier | AnnotatedNode)** A node of type Syntax.Identifier
-   `newName` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The new name of the identifier

#### as_source

[src/ast.js:348-366](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L348-L366 "Source code on GitHub")

Return the AST nodes as source code, including any modifications made.

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** The lossy source code

#### get_from_function

[src/ast.js:374-489](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/ast.js#L374-L489 "Source code on GitHub")

**Parameters**

-   `node` **(FunctionDeclaration | FunctionExpression | MethodDefinition | ArrowFunctionExpression | Property | [Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling))** 
-   `whatToGet` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)**  (optional, default `'all'`)

Returns **([Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling)> | [string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String) | CFGInfo)** 

### 

[src/leader.js:8-8](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/leader.js#L8-L8 "Source code on GitHub")

Describe what cfg does.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### 

[src/dump.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/dump.js#L7-L7 "Source code on GitHub")

Functions to dump out information as pretty CLI tables.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### 

[src/manager.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L7-L7 "Source code on GitHub")

The class that manages the individual blocks.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### BlockManager

[src/manager.js:21-330](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L21-L330 "Source code on GitHub")

**Parameters**

-   `ast` **[AST](#ast)** 
-   `options` **CFGOptions** 

#### blocks

[src/manager.js:33-33](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L33-L33 "Source code on GitHub")

#### forEach

[src/manager.js:81-84](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L81-L84 "Source code on GitHub")

**Parameters**

-   `fn` **function ([CFGBlock](#cfgblock), [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number), [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>): any** 

#### map

[src/manager.js:89-92](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L89-L92 "Source code on GitHub")

**Parameters**

-   `fn` **function ([CFGBlock](#cfgblock), [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number), [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>): any** 

#### get

[src/manager.js:98-101](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L98-L101 "Source code on GitHub")

**Parameters**

-   `index` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 

Returns **[CFGBlock](#cfgblock)** 

#### toString

[src/manager.js:124-127](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L124-L127 "Source code on GitHub")

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### toTable

[src/manager.js:132-135](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L132-L135 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>>** 

#### iterator

[src/manager.js:140-147](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L140-L147 "Source code on GitHub")

#### create_dot

[src/manager.js:153-178](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/manager.js#L153-L178 "Source code on GitHub")

**Parameters**

-   `title` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

### 

[src/dot.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/dot.js#L7-L7 "Source code on GitHub")

Create a DOT file for graphviz.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### 

[src/block.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L7-L7 "Source code on GitHub")

CFG block definition, equivalent to a basic block in compiler parlance.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### CFGBlock

[src/block.js:32-466](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L32-L466 "Source code on GitHub")

**Parameters**

-   `id` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 
-   `edges` **[Edges](#edges)** 

#### nodes

[src/block.js:46-46](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L46-L46 "Source code on GitHub")

#### succesors_as_indices

[src/block.js:61-64](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L61-L64 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)>** 

#### successors

[src/block.js:69-72](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L69-L72 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>** 

#### succs

[src/block.js:77-80](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L77-L80 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>** 

#### preds

[src/block.js:85-88](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L85-L88 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>** 

#### isEmpty

[src/block.js:93-96](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L93-L96 "Source code on GitHub")

Returns **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** 

#### classify

[src/block.js:103-107](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L103-L107 "Source code on GitHub")

**Parameters**

-   `to` **([number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number) \| [CFGBlock](#cfgblock))** 
-   `type` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

Returns **[CFGBlock](#cfgblock)** 

#### follows

[src/block.js:113-123](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L113-L123 "Source code on GitHub")

**Parameters**

-   `cb` **([CFGBlock](#cfgblock) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>)** 

Returns **[CFGBlock](#cfgblock)** 

#### from

[src/block.js:129-132](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L129-L132 "Source code on GitHub")

**Parameters**

-   `cb` **([CFGBlock](#cfgblock) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>)** 

Returns **[CFGBlock](#cfgblock)** 

#### to

[src/block.js:138-150](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L138-L150 "Source code on GitHub")

**Parameters**

-   `cb` **([CFGBlock](#cfgblock) \| [Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>)** 

Returns **[CFGBlock](#cfgblock)** 

#### remove_succs

[src/block.js:155-159](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L155-L159 "Source code on GitHub")

Returns **[CFGBlock](#cfgblock)** 

#### remove_succ

[src/block.js:165-169](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L165-L169 "Source code on GitHub")

**Parameters**

-   `kill` **([number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number) \| [CFGBlock](#cfgblock))** 

Returns **[CFGBlock](#cfgblock)** 

#### as

[src/block.js:175-187](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L175-L187 "Source code on GitHub")

**Parameters**

-   `nodeType` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 

Returns **[CFGBlock](#cfgblock)** 

#### edge_as

[src/block.js:196-202](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L196-L202 "Source code on GitHub")

Sets the last edge to type.

**Parameters**

-   `edgeType` **[Edge](#edge)** 
-   `to` **([number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number) \| [CFGBlock](#cfgblock))?** 

Returns **[CFGBlock](#cfgblock)** 

#### not

[src/block.js:210-214](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L210-L214 "Source code on GitHub")

Removes a type from this block.

**Parameters**

-   `nodeType` **[Edge](#edge)** 

Returns **[CFGBlock](#cfgblock)** 

#### whenTrue

[src/block.js:222-229](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L222-L229 "Source code on GitHub")

For test nodes, this adds the edge taken when the condition is true.

**Parameters**

-   `block` **[CFGBlock](#cfgblock)** 

Returns **[CFGBlock](#cfgblock)** 

#### whenFalse

[src/block.js:237-244](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L237-L244 "Source code on GitHub")

For test nodes, this adds the edge taken when the condition is false.

**Parameters**

-   `block` **[CFGBlock](#cfgblock)** 

Returns **[CFGBlock](#cfgblock)** 

#### add

[src/block.js:252-258](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L252-L258 "Source code on GitHub")

Add a current-level AST node to this block.

**Parameters**

-   `node` **(AnnotatedNode | BaseNode | [Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling))** 

Returns **[CFGBlock](#cfgblock)** 

#### first

[src/block.js:265-268](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L265-L268 "Source code on GitHub")

Returns the first AST node (if any) of this block.

Returns **(AnnotatedNode | BaseNode | [Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling))?** 

#### last

[src/block.js:275-278](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L275-L278 "Source code on GitHub")

Returns the last AST node (if any) of this block.

Returns **(AnnotatedNode | BaseNode | [Node](https://developer.mozilla.org/docs/Web/API/Node/nextSibling))?** 

#### by

[src/block.js:286-290](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L286-L290 "Source code on GitHub")

Free-text field indicating the manner of of creation of this node. For information in graphs and printouts only.

**Parameters**

-   `txt` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

Returns **[CFGBlock](#cfgblock)** 

#### isa

[src/block.js:298-301](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L298-L301 "Source code on GitHub")

Check if this block has a particular type.

**Parameters**

-   `typeName` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 

Returns **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** 

#### eliminate

[src/block.js:308-316](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L308-L316 "Source code on GitHub")

Remove itself if it's an empty node and isn't the start or exit node.

Returns **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** true if it was deleted

#### defer_edge_type

[src/block.js:321-324](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L321-L324 "Source code on GitHub")

**Parameters**

-   `type` **[Edge](#edge)** 

#### graph_label

[src/block.js:337-347](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L337-L347 "Source code on GitHub")

For the vertices.

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### lines

[src/block.js:354-378](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L354-L378 "Source code on GitHub")

Stringified line numbers for this block.

Returns **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### pred_edge_types

[src/block.js:383-386](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L383-L386 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** 

#### succ_edge_types

[src/block.js:391-394](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L391-L394 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** 

#### split_by

[src/block.js:401-414](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L401-L414 "Source code on GitHub")

**Parameters**

-   `arr` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;any>** 
-   `chunkSize` **[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)** 

Returns **ArrayArray&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** 

#### toRow

[src/block.js:421-437](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L421-L437 "Source code on GitHub")

Headers are
TYPE / LINES / LEFT EDGES / NODE / RIGHT EDGES / CREATED BY / AST

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** 

#### toString

[src/block.js:442-464](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/block.js#L442-L464 "Source code on GitHub")

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)>** 

### 

[src/edges.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L7-L7 "Source code on GitHub")

Handles all edges.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### Edges

[src/edges.js:51-409](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L51-L409 "Source code on GitHub")

Keeps track of edges apart from the successors listed in the [CFGBlock](#cfgblock) so we can
attach types and classifications on edges separate from their blocks.

**Parameters**

-   `bm` **[BlockManager](#blockmanager)** 

#### reindex

[src/edges.js:82-96](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L82-L96 "Source code on GitHub")

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

Returns **[Edges](#edges)** 

#### add

[src/edges.js:106-121](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L106-L121 "Source code on GitHub")

Add an edge between to CFGBlocks.

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `to` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `type` **[Edge](#edge)**  (optional, default `Edge.NONE`)

Returns **[Edges](#edges)** 

#### classify

[src/edges.js:131-140](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L131-L140 "Source code on GitHub")

Set a type on an arbitrary edge.

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `to` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `ctype` **[Edge](#edge)** 

Returns **[Edges](#edges)** 

#### not

[src/edges.js:150-158](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L150-L158 "Source code on GitHub")

Remove a type from an arbitrary edge.

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `to` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `type` **[Edge](#edge)** 

Returns **[Edges](#edges)** 

#### retarget_multiple

[src/edges.js:218-231](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L218-L231 "Source code on GitHub")

Point one or more edges to a new [CFGBlock](#cfgblock), used in block removal.

**Parameters**

-   `node` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

Returns **[Edges](#edges)** 

#### remove_succ

[src/edges.js:240-258](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L240-L258 "Source code on GitHub")

Remove a successor [CFGBlock](#cfgblock) from a [CFGBlock](#cfgblock)

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `to` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

Returns **[Edges](#edges)** 

#### get_succs

[src/edges.js:266-273](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L266-L273 "Source code on GitHub")

Get all successors for a given [CFGBlock](#cfgblock).

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>** 

#### get_preds

[src/edges.js:281-288](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L281-L288 "Source code on GitHub")

Get all predecessors for a given [CFGBlock](#cfgblock)

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[CFGBlock](#cfgblock)>** 

#### renumber

[src/edges.js:295-318](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L295-L318 "Source code on GitHub")

Renumber all indices (`id` field) because of removed [CFGBlock](#cfgblock)s.

**Parameters**

-   `newOffsets` **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)>** 

#### successors

[src/edges.js:347-350](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L347-L350 "Source code on GitHub")

#### has

[src/edges.js:359-367](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L359-L367 "Source code on GitHub")

Is there an edge of the gievn type?

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 
-   `type` **[Edge](#edge)** 

Returns **[boolean](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Boolean)** 

#### edges

[src/edges.js:375-380](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L375-L380 "Source code on GitHub")

Get edge information for a given [CFGBlock](#cfgblock), i.e. successors.

**Parameters**

-   `from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

#### preds

[src/edges.js:387-393](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L387-L393 "Source code on GitHub")

Get all predecessors for a given [CFGBlock](#cfgblock)

Returns **[Array](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Array)&lt;[number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number)>** 

#### pred_edges

[src/edges.js:401-408](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/edges.js#L401-L408 "Source code on GitHub")

Get all predecessor edge information for a given [CFGBlock](#cfgblock).

**Parameters**

-   `_from` **([CFGBlock](#cfgblock) \| [number](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Number))** 

### 

[src/visitors.js:156-156](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/visitors.js#L156-L156 "Source code on GitHub")

#### The `for` Statement

The `ForStatement` works like shown in the following. Note that the init, test, and update
may be absent, leaving only a body, in which case it is an infirnite loop, barring a `break`
or `continue`.

     PREV ->
                     ┌───────────────┐
                     v               │
          INIT -> TEST -> BODY -> UPDATE  <───── `ContinueStatement` target
                    │
                    └────> REST OF CODE PAST LOOP   <───── `BreakStatement` target

     [INIT]    ->    [TEST]

     BODY    -> [UPDATE]

     REST

#### The `if` Statement

           │
           │
           V
     ┌────────────┐     ┌─────────────┐
     │    test    │ ──> │ (alternate) │
     └────────────┘     └─────────────┘
           │                   │
           │                   │
           V                   │
     ┌────────────┐            │
     │ consequent │            │
     └────────────┘            │
           │                   │
           │                   │
           V                   │
     ┌────────────┐            │
     │   block    │<───────────┘
     └────────────┘

#### The `switch` Statement

Originally, I treated the Switc/SwitchCase structure like shown in the
first diagram but ended up changing it to something more leaborate but
closer to how it would actually be treated by a compiler (sub-dividing
test cases and so on).

             ┌──────────────┐
             │              │
             │    switch    │
             │              │
             └──┬─┬─┬─┬─┬─┬─┘
                │ │ │ │ │ │
                │ │ │ │ │ │         ┌─────────────┐
                │ │ │ │ │ │         │             │
                │ │ │ │ │ └────────>│    case1    │
                │ │ │ │ │           │             │
                │ │ │ │ │           └─────────────┘
                │ │ │ │ │
                │ │ │ │ │           ┌─────────────┐
                │ │ │ │ │           │             │
                │ │ │ │ └──────────>│    case2    │
                │ │ │ │             │             │
                │ │ │ │             └─────────────┘
                │ │ │ │
                │ │ │ │             ┌─────────────┐
                │ │ │ │             │             │ [ fall through succ. is next case ]
                │ │ │ └────────────>│    case3    │
                │ │ │               │             │
                │ │ │               └──────┬──────┘
                │ │ │                      │
                │ │ │                Falls through
                │ │ │                      │
                │ │ │               ┌──────┴──────┐
                │ │ │               │             │ [ previous falls through, preds are switch and previous case ]
                │ │ └──────────────>│    case4    │
                │ │                 │             │
                │ │                 └─────────────┘
                │ │
                │ │                 ┌─────────────┐
                │ │                 │             │
                │ └────────────────>│   default   │
                │                   │             │
       Pred if no default           └──────┬──────┘
                │                          │
                v                    Pred if default
                ┌─────────────┐            │
                │             │            │
                │    next     │<───────────┘
                │             │
                └─────────────┘

Now, I deal with it as shown below. We go from test to test, based on the discriminant
in the `SwitchStatement` itself. A `false` result moves on to the next test while a
true result transfers control to the body of the `SwitchCase`. Some caveats are:

1.  Without a `BreakStatement` at the end that post-dmoniates the block, it will fall
    through to the next `SwitchCase` body.
2.  There is no requirement that a `SwitchCase` will have a body block, in which case, we
    keep falling to the next one, until we reach a body or exit the `SwitchStatement` entirely.
3.  We go from false test to false test, however, there may be one `SwitchCase1 that doesn't
    have a test, the default case. The default case need not be at the end and follow the
    same rules as described above regarding falling through. The false edge from the final
    test will go to the default body, if there is one, or the normal block following the`SwitchStatement\`, if there is one (otherwise it will eventually work its way up to the
    exit node).

      TEST -> true -> body -> break -> out
                           \-> no break -> next body

           -> false -> next test

if last

     TEST -> true -> body -> break -> out
                          -> no break -> out

          -> false -> default OR out



                 consequent
     SWITCH           │
        │             v
        └──> TEST ───────────> BODY ─────────┐ break
               │                │            │
            alt│              no│break       │
               │              no│body        │
               │                │            │
               V                V            │
             TEST ───────────> BODY ─────────┤ break
               │                │            │
            alt│              no│break       │
               │              no│body        │
               │                │            │
               │                V            │
               │  DEFAULT ───> BODY ─────────┤ break (or not)
               │     ^          │            │
               │     │        no│break       │
               │   if│false   no│body        │
               │  and│default   │            │
               V     │          V            │
             TEST ───────────> BODY ─────────┤ break (or not)
                   if│false                  │
               and no│default                │
                     v                       │
     CONT <──────────────────────────────────┘

**Meta**

-   **version**: 1.0.0
    \*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*\*
-   **author**: julian on 12/22/17

### 

[src/types.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/types.js#L7-L7 "Source code on GitHub")

Think types.h

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### Block

[src/types.js:28-42](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/types.js#L28-L42 "Source code on GitHub")

### Edge

[src/types.js:52-68](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/types.js#L52-L68 "Source code on GitHub")

### defaultOutputOptions

[src/types.js:77-87](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/types.js#L77-L87 "Source code on GitHub")

The default display options for table and string output.

### output

[src/types.js:96-99](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/types.js#L96-L99 "Source code on GitHub")

Override display options.

**Parameters**

-   `options`  

### 

[src/utils.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/utils.js#L7-L7 "Source code on GitHub")

* * *

\` \* @file Describe what utils does.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### pluginManager

[src/utils.js:15-15](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/utils.js#L15-L15 "Source code on GitHub")

### 

[src/plugins.js:7-7](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/plugins.js#L7-L7 "Source code on GitHub")

Manage all the plugins.

**Meta**

-   **since**: 1.0.0
-   **author**: Julian Jensen &lt;jjdanois@gmail.com>

### callback

[src/plugins.js:131-157](https://github.com/julianjensen/ast-flow-graph/blob/7ec32330453d0a366d208916ee16c0c78bb75efe/src/plugins.js#L131-L157 "Source code on GitHub")

**Parameters**

-   `topKey` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `subKey` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)?** 
-   `args` **...any** 

Returns **any** 

## License

MIT © [Julian Jensen](https://github.com/julianjensen/ast-flow-graph)

[coveralls-url]: https://coveralls.io/github/julianjensen/ast-flow-graph?branch=master

[coveralls-image]: https://coveralls.io/repos/github/julianjensen/ast-flow-graph/badge.svg?branch=master

[travis-url]: https://travis-ci.org/julianjensen/ast-flow-graph

[travis-image]: http://img.shields.io/travis/julianjensen/ast-flow-graph.svg

[depstat-url]: https://gemnasium.com/github.com/julianjensen/ast-flow-graph

[depstat-image]: https://gemnasium.com/badges/github.com/julianjensen/ast-flow-graph.svg

[npm-url]: https://badge.fury.io/js/ast-flow-graph

[npm-image]: https://badge.fury.io/js/ast-flow-graph.svg

[license-url]: https://github.com/julianjensen/ast-flow-graph/blob/master/LICENSE

[license-image]: https://img.shields.io/badge/license-MIT-brightgreen.svg

[snyk-url]: https://snyk.io/test/github/julianjensen/ast-flow-graph

[snyk-image]: https://snyk.io/test/github/julianjensen/ast-flow-graph/badge.svg

[david-dm-url]: https://david-dm.org/julianjensen/ast-flow-graph

[david-dm-image]: https://david-dm.org/julianjensen/ast-flow-graph.svg
